@@ -153,7 +153,10 @@ class Detector(object):
         assert not len(restypes) == 0,'BUG: no valid type could be detected'
 
         if len(restypes) > 1:
-            raise RuntimeError, 'Could not decide between types: ' + str(restypes)
+            if not all([r.has_missing for r in restypes]):
+                raise RuntimeError, 'Could not decide between types: ' + str(restypes)
+            else:
+                return rtypes.TypeAny(has_missing=True);
         return restypes[0]
 
     def _findAcceptableDescendants(self, seq, scanner=None):
@@ -298,7 +301,10 @@ class FixedDimRep(DimRep):
             self.lengths = [] 
         else:
             self.length_type = LENGTH_FIXED
-            self.lengths = dim.shape
+            if isinstance(dim.shape,int):
+                self.lengths = dim.shape
+            else:
+                self.lengths = UNDEFINED
             #FIXME: give lengths of variable dimensions from data, will also enable repeat var dimensions
         
         self.nparents = 0
@@ -828,35 +834,101 @@ class StringIntScanner(StringRealScanner):
         return res
 registerTypeScanner(StringIntScanner)
 
-class StringDNAScanner(StringScanner):
+class StringProteinScanner(StringScanner):
     need_convert=False
     parentcls=StringScanner
-    regmatch = re.compile('^[acgtnACGTN]+$')
-    
+    regmatch = re.compile('^[AaBbCcDdEeFfGgHhIiKkLlMmNnOoPpQqRrSsTtUuVvWwYyZzXx\*\-]+$')
+    ntype = rtypes.TypeProteinSequence
+
+    def __init__(self, detector):
+        StringScanner.__init__(self, detector);
+        self.has_missing = False;
+        self.evidence = 0
+        self.nelem = 0
+
     def unregister(self, create_parent=False):
-        res = super(StringDNAScanner,self).unregister(create_parent)
+        res = super(StringProteinScanner,self).unregister(create_parent)
         if create_parent:
             res.max_nchars = self.max_nchars
         return res
 
     def getType(self):
-        ntype = rtypes.TypeDNASequence
+        if not self.nelem or self.evidence / self.nelem <= 25:
+            if self.nelem:
+                util.info('Detecting possible protein sequences as string type due to short averaged length (%d).\n' % (self.evidence / self.nelem))
+            return StringScanner.getType(self)
+
+        ntype = self.ntype
         d = dimensions.Dim(UNDEFINED, (True,) * len(self.getDimReps(0)), self.detector.hasMissing())
         dims = dimpaths.DimPath(d)
-        return ntype(self.detector.hasMissing(), dims)
+        return ntype(self.detector.hasMissing() or self.has_missing, dims)
 
 
+    def scan(self, seq):
+        res = StringScanner.scan(self, seq)
+        rm = self.regmatch
+        if res:
+            minsize = numpy.inf
+            minseq  = ""
+            for elem in seq.ravel():
+                if not elem:
+                    self.has_missing = True;
+                    continue
+                if rm.match(elem) is None:
+                    return False
+                
+                size = len(elem)
+                if size:
+                    self.evidence += size
+                    self.nelem += 1
+
+        if minsize < 25:
+            return False
+        return res
+registerTypeScanner(StringProteinScanner);
+
+
+class StringDNAScanner(StringProteinScanner):
+    need_convert=False
+    parentcls=StringProteinScanner
+    regmatch = re.compile('^[acgtnACGTNurykmswbdhvnx\-URYKMSWBDHVNX]+$')
+    ntype = rtypes.TypeDNASequence
+    
+    def unregister(self, create_parent=False):
+        res = super(StringDNAScanner,self).unregister(create_parent)
+        if create_parent:
+            res.has_missing = self.has_missing
+            res.evidence = self.evidence
+            res.nelem = self.nelem
+        return res
+   
+    def getType(self):
+        if not self.nelem:
+            return StringScanner.getType(self)
+
+        ntype = self.ntype
+        d = dimensions.Dim(UNDEFINED, (True,) * len(self.getDimReps(0)), self.detector.hasMissing())
+        dims = dimpaths.DimPath(d)
+        return ntype(self.detector.hasMissing() or self.has_missing, dims)
+
     def scan(self, seq):
         res = StringScanner.scan(self, seq)
         rm = self.regmatch
         if res:
             for elem in seq.ravel():
                 if not elem:
+                    self.has_missing = True;
                     continue
                 if rm.match(elem) is None:
                     return False
+                
+                size = len(elem)
+                if size:
+                    self.evidence += size
+                    self.nelem += 1
         return res
-#registerTypeScanner(StringDNAScanner)
+registerTypeScanner(StringDNAScanner)
+
 
 
 class SliceScanner(TypeScanner):
