@@ -68,6 +68,12 @@ class DimPath(tuple):
     def __add__(self, other):
         return DimPath(*tuple.__add__(self,other))
 
+    def startsWithRoot(self):
+        return self and self[0] is root
+
+    def endsWithEnd(self):
+        return self and self[-1] is end
+
     def strip(self):
         if(self and self[0] is root):
             if(self[-1] is end):
@@ -121,15 +127,15 @@ class DimPath(tuple):
             newpath.append(dim)
         return DimPath(*newpath)#}}}
     
-    def removeDim(self, pos, elem_specifier=None, subtype=None):#{{{
+    def removeDim(self, pos, elem_specifier=None, subtype=None, has_missing=False):#{{{
         ndims = []
         for p in xrange(max(pos + 1,0), len(self)):
-            r = self[p].removeDepDim(p - pos - 1, elem_specifier)
+            r = self[p].removeDepDim(p - pos - 1, elem_specifier, has_missing)
             ndims.append(r)
 
         res = self[:max(pos,0)] + DimPath(*ndims)
         if(not subtype is None):
-            subtype = subtype._removeDepDim(pos=pos - len(self), elem_specifier=elem_specifier)
+            subtype = subtype._removeDepDim(pos=pos - len(self), elem_specifier=elem_specifier, has_missing=has_missing)
             return (res,subtype)
         else:
             return res#}}}
@@ -817,18 +823,19 @@ def extendParentDim(path, sourcepaths, length=1):#{{{
             r = spath[:(lastpos + 1 - len(path))]
             xpaths.add(r)
         
-    while(all([len(xpath) > 0 for xpath in xpaths])and (len(ndims) < length or length == -1)):
+    while((len(ndims) < length or length == -1)):
         xdims = set()
         nxpaths = set()
         for xpath in xpaths:
-           xdims.add(xpath[-1])
-           nxpaths.add(xpath[:-1])
+           if xpath:
+               xdims.add(xpath[-1])
+               nxpaths.add(xpath[:-1])
         
         if(not xdims or len(xdims) > 1):
             if length == -1:
                 break
             else:
-               raise RuntimeError, "Cannot find unique parent for dim: " + str(path[0])
+                raise RuntimeError, "Cannot find unique parent for dim: " + str(path[0])
         xpaths = nxpaths
         ndims.append(xdims.pop())
 
@@ -836,6 +843,23 @@ def extendParentDim(path, sourcepaths, length=1):#{{{
     return path
     #}}}
 
+
+def appendChildDim(path, sourcepaths):
+    xdims = set()
+    util.debug_here()
+    for spath in sourcepaths:
+        lastposs = spath.matchDimPath(path)
+        for lastpos in lastposs:
+            if len(spath) > (lastpos+1):
+                xdims.add(spath[lastpos+1])
+    print xdims 
+    if len(xdims) != 1:
+        return False
+
+    npath = list(path)
+    npath.append(xdims.pop)
+    return DimPath(*npath)
+
 def getArrayDimPathFromType(rtype):
     if(rtype.__class__ is rtypes.TypeArray):
         return rtype.dims + getArrayDimPathFromType(rtype.getSubType())
