@@ -370,7 +370,7 @@ class FilterOp(UnaryUnaryOp):#{{{
 
         sdims = stype.dims
         if(ndim is None):
-            sdims, subtype = sdims.removeDim(0, refconstraint, stype.subtypes[0])
+            sdims, subtype = sdims.removeDim(0, refconstraint, stype.subtypes[0], sdims[0].has_missing)
         else:
             sdims, subtype = sdims.updateDim(0, ndim, stype.subtypes[0])
         
@@ -475,7 +475,7 @@ class FlatDimOp(UnaryUnaryOp):#{{{
     def __init__(self,slice,flatpos, ndim):
         stype = slice.type
         sdims = slice.dims
-        sdims,stype = sdims.removeDim(flatpos,None,stype)
+        sdims,stype = sdims.removeDim(flatpos,None,stype,sdims[flatpos].has_missing)
         sdims,stype = sdims.updateDim(flatpos-1,ndim,stype)
 
         self.flatpos = flatpos
@@ -582,7 +582,8 @@ class UnaryFuncAggregateOp(UnaryFuncOp):#{{{
     
     def __init__(self, funcname, sig, name, dtype, packdepth, slice, **kwargs):
         self.packdepth = packdepth
-        sdims, ntype = slice.dims.removeDim(len(slice.dims) - packdepth, (funcname,name,dtype, len(slice.dims) - packdepth), dtype)
+        remdimpos = len(slice.dims) - packdepth
+        sdims, ntype = slice.dims.removeDim(remdimpos, (funcname,name,dtype, remdimpos), dtype, slice.dims[remdimpos].has_missing)
 
         UnaryFuncOp.__init__(self, slice, funcname, sig, name, dtype, dims=sdims, **kwargs)
         self.type = ntype
@@ -598,13 +599,13 @@ class MultiUnaryOp(UnaryOp):#{{{
 
 class EachOp(MultiUnaryOp):
     __slots__ = ['named_params', 'eachfunc','extra_params']
-    def __init__(self, source_slices, eachfunc, rtype=rtypes.unknown, named_params=False, extra_params={}):
+    def __init__(self, source_slices, eachfunc, rtype=rtypes.unknown, named_params=False, keep_name=False, extra_params={}):
         nbookmarks = reduce(operator.__or__, [s.bookmarks for s in source_slices])
         for slice in source_slices[1:]:
             assert slice.dims == source_slices[0].dims, 'Unequal dims'
         dims = source_slices[0].dims
 
-        if not isinstance(eachfunc, context.Context) and hasattr(eachfunc,'func_name') and not eachfunc.func_name=='<lambda>':
+        if not keep_name and not isinstance(eachfunc, context.Context) and hasattr(eachfunc,'func_name') and not eachfunc.func_name=='<lambda>':
             name = util.valid_name(eachfunc.func_name)
         else:
             name = '_'.join([s.name for s in source_slices])
@@ -794,5 +795,31 @@ class EquiJoinIndexOp(MultiMultiOp):#{{{
         MultiMultiOp.__init__(self, (leftslice,rightslice),(r1,r2))#}}}
 
 
+class BlastIndexOp(MultiMultiOp):#{{{
+    __slots__ = ["blast_type", "folder", "reciprocal", "normalize", "overwrite", "blastopts"]
+
+    def __init__(self, leftslice, rightslice,  blast_type, folder, reciprocal = True, normalize = False, overwrite = False, blastopts='' ):
+        assert leftslice.dims == rightslice.dims, "Parent dims of join fields should be equal"
+        if leftslice.type.dims[0].name == rightslice.type.dims[0].name:
+            name= leftslice.type.dims[0].name
+        else:
+            name= leftslice.type.dims[0].name + "_" + rightslice.type.dims[0].name
+        ndim = dimensions.Dim(UNDEFINED, (True,) * len(leftslice.dims), name=name)
+
+        types = (rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypePlatformInt(),  rtypes.TypePlatformInt(), rtypes.TypePlatformInt(), rtypes.TypeReal64(), rtypes.TypeReal64(), rtypes.TypeReal64())
+        names = ("qseqid", "sseqid", "qlen", "qstart", "qend", "slen", "sstart", "send", "length", "mismatch", "gapopen", "pident", "evalue", "bitscore" ) 
+        slice_types = [ rtypes.TypeArray(subtypes=(t,), dims=dimpaths.DimPath(ndim)) for t in types ];
+
+        r = tuple([ SelectOp(self, i, names[i], slice_types[i] ,leftslice.dims, set([])) for i in xrange(len(slice_types)) ]);
+        
+        self.blast_type = blast_type
+        self.folder = folder
+        self.reciprocal = reciprocal
+        self.normalize = normalize
+        self.overwrite = overwrite
+        self.blastopts= blastopts
+
+        MultiMultiOp.__init__(self, (leftslice,rightslice),r)#}}}
+
 
 
