@@ -33,7 +33,7 @@ class UnpackArray(repops.UnaryOpRep):
                     slice = ops.UnpackArrayOp(slice,ndim=ndim)
             nslices.append(slice)
         if not unpacked:
-            raise AttributeError, 'No array-typed slice with outer dimension name "' + name + '" to unpack'
+            raise AttributeError, 'No array-typed slice with outer dimension name "' + str(name) + '" to unpack'
         return self._initialize(tuple(nslices))
 
 class DimRename(repops.UnaryOpRep):#{{{
@@ -229,6 +229,7 @@ class FlatAll(repops.UnaryOpRep):
 class Flat(repops.UnaryOpRep):
     def _sprocess(self,source,name=None,dim=-1):
         selpath = dimpaths.identifyUniqueDimPathSource(source,dim)
+
         nslices = self._apply(source._slices, selpath, name)
 
         return self._initialize(tuple(nslices))
@@ -236,7 +237,8 @@ class Flat(repops.UnaryOpRep):
     @classmethod
     def _apply(cls, fslices, selpath, name=None):
         #create new merged dimension
-        selpath = dimpaths.extendParentDim(selpath,[s.dims for s in fslices],max(2,1 + len(selpath[-1].dependent)))
+        selpath = dimpaths.extendParentDim(selpath, [s.dims for s in fslices], max(2,1 + len(selpath[-1].dependent)))
+        assert len(selpath) >= 2, 'Could not find unique parent dimension for %s. Please specify one of the slices that needs to be flattened.' % str(selpath)
         
         #determine new dim
         if(selpath[-1].shape != UNDEFINED and selpath[-2].shape != UNDEFINED):
