@@ -38,34 +38,57 @@ class RequestUnaryOpRep(repops.UnaryOpRep):
 class To(RequestUnaryOpRep):
     def _sprocess(self, source, *slicesel, **kwargs):
         do = kwargs.pop("Do")
-        assert not kwargs, "Unknown parameters: " + str(kwargs)
+        separate = kwargs.pop('separate',False)
 
+        assert not kwargs, "Unknown parameters: " + str(kwargs)
+    
         if not self._req_sources:
             for ssel in slicesel:
                 r = source.Get(ssel) 
-                #assert len(r._slices) == 1, "To action can only be applied to single slices"
                 all_pos = []
                 for slice in r._slices:
-                    #slice = r._slices[0]
-                    assert slice in source._slices, "Selected slice in to should not be operated upon"
+                    assert slice in source._slices, "Selected slice in To operation should only be operated on using Do argument"
                     pos = source._slices.index(slice)
                     all_pos.append(pos)
-
-                if(isinstance(do, context.Context)):
-                    r = context._apply(do, r)
+                if separate:
+                    for i in range(len(r._slices)):
+                        z = r.Get(i)
+                        if(isinstance(do, context.Context)):
+                            z = context._apply(do, z)
+                        else:
+                            z = do(z)
+                        self._req_sources.append(([all_pos[i]],z))
                 else:
-                    r = do(r)
+                    if(isinstance(do, context.Context)):
+                        r = context._apply(do, r)
+                    else:
+                        r = do(r)
 
-                self._req_sources.append((all_pos, r))
+                    self._req_sources.append((all_pos, r))
+        
+        if not all([elem[1]._slicesKnown() for elem in self._req_sources]):
+            return
 
         nslices = list(source._slices)
-        for all_pos, r in self._req_sources:
-            if not r._slicesKnown():
-                return 
-            for slice,pos in zip(r._slices, all_pos):
-                nslices[pos] = slice
 
-        self._initialize(tuple(nslices))
+        index = {}
+        for all_pos, r in self._req_sources:
+            for p in all_pos:
+                index[p] = (all_pos, r._slices)
+
+        nnslices = []
+        for pos, nslice in enumerate(nslices):
+            if pos in index:
+                all_pos, r = index[pos]
+                if len(all_pos) == 1:
+                    nnslices.extend(r)
+                else:
+                    assert len(all_pos) == len(r), 'Number of source and result slices in a To/Do operation should match when number of source slices > 1'
+                    nnslices.append(r[all_pos.index(pos)])
+            else:
+                nnslices.append(nslice)
+
+        self._initialize(tuple(nnslices))
        
 
 class Project(RequestUnaryOpRep):
@@ -157,7 +180,7 @@ class Project(RequestUnaryOpRep):
         
         if(not self._req_sources):
             self._req_sources = self._getSliceList(chain(zip([None] * len(args),args),kwds.iteritems()), [], cur_slices)
-       
+        
         nslices = self._toSlices(self._req_sources)
         if nslices is None:
             return None
@@ -420,13 +443,16 @@ class Each(repops.UnaryOpRep):
     _ocls = ops.PackTupleOp
 
     def _sprocess(self, source, **kwargs):
-        nslice = self._apply(source._slices, **kwargs)
-        return self._initialize((nslice,))
+        nslices = self._apply(source._slices, **kwargs)
+        return self._initialize(nslices)
 
     @classmethod
-    def _apply(cls, slices, eachfunc, dtype=rtypes.unknown, named_params=False, **kwargs):
-        slices,plans = ops.broadcast(slices, mode='dim', partial=False)
+    def _apply(cls, slices, eachfunc, dtype=rtypes.unknown, named_params=False, keep_name=False, per_slice=False, **kwargs):
         if(not isinstance(dtype,rtypes.TypeUnknown)):
             dtype = rtypes.createType(dtype,len(slices[0].dims)) 
-        return ops.EachOp(slices, eachfunc, dtype, named_params, kwargs)
+        if per_slice:   
+            return tuple([ops.EachOp((slice,), eachfunc, dtype, named_params, keep_name, kwargs) for slice in slices])
+        else:
+            slices,plans = ops.broadcast(slices, mode='dim', partial=False)
+            return (ops.EachOp(slices, eachfunc, dtype, named_params, keep_name, kwargs),)
 
