@@ -98,6 +98,9 @@ class MissingType(object):#{{{
     def __rxor__(self, other):
         return self
 
+    def __pow__(self, value):
+        return self
+
     def __eq__(self, other):
         return self
 
@@ -128,6 +131,15 @@ class MissingType(object):#{{{
     def __abs__(self):
         return self
 
+    def log(self):
+        return self
+    
+    def log2(self):
+        return self
+
+    def log10(self):
+        return self
+
     def __nonzero__(self):
         return False
     
