@@ -20,6 +20,8 @@ import csv
 from logging import error,warning,info,debug
 import random
 import re
+import time
+import shlex, subprocess, os;
 
 def resplit(str, sep=' ', sc=[]):
     if sc == []:
@@ -28,8 +30,11 @@ def resplit(str, sep=' ', sc=[]):
     containers = "";
     for c in sc:
         containers = containers + "|%s[^%s]*%s" %(c, c, c) ;
+
+    if isinstance(sep, (tuple,list)):
+        sep = "".join(sep)
   
-    pat = "%s(?=(?:[^%s]%s)*$)" % (sep, sc, containers);
+    pat = "[%s](?=(?:[^%s]%s)*$)" % (sep, sc, containers);
   
     return re.split(pat, str);
 
@@ -76,18 +81,24 @@ def save_rep(r, filename):
     s = cPickle.dumps(r,protocol=2)
     s = zlib.compress(s)
     f.write(s)
+    f.close()
 
-
-def save_csv(r, filename):
+def save_csv(r, filename, remove_line_end=True, names=True):
     f = open(filename,'wb')
     r= r.Array(tolevel=1)
-    data = r.Cast(str).Tuple().ToPython();
+    data = r.Cast(str)
     if filename.endswith('tsv'):
-        w = csv.writer(f,delimiter='\t');
+        w = csv.writer(f,delimiter='\t', quotechar='"', quoting=csv.QUOTE_MINIMAL);
     else:
-        w = csv.writer(f);
-    w.writerow(r.Names);
+        w = csv.writer(f, quotechar='"', quoting=csv.QUOTE_MINIMAL);
+
+    if remove_line_end:
+        data = data.Each(lambda x: x.replace('\n',''), dtype=str, per_slice=True)
+    data = data.Tuple().ToPython();
+    if names:
+        w.writerow(r.Names);
     w.writerows(data);
+    f.close()
 
 def load_rep(filename):
     f = open(filename, 'rb')
@@ -539,6 +550,12 @@ def seq_names(n, exclude=set()):
             break
     return rn
 
+def gen_seq_names(exclude=set()):
+    for i in seqgen():
+        z = 'd' + str(i)
+        if not z in exclude:
+            yield z
+
 def uniqify_names(names, exclude=set()):
     rn = seq_names(len(names), exclude=exclude)
 
@@ -563,4 +580,144 @@ def append_name(name, exclude):
         if not z in exclude:
             return z
         
-   
+###############################################################################
+
+def run_par_cmds(cmd_list, max_threads=12, stdin=None, stdout=None, stderr=None):
+
+  p = [];
+  i = 0;
+  retval = 0;
+  cmds = len(cmd_list);
+
+  while True:
+    while len(p) < max_threads and i < cmds:
+      print "RUNNING: %s" % cmd_list[i]; sys.stdout.flush();
+      p.append( (run_cmd(cmd_list[i], bg=True, stdin=stdin, stdout=stdout, stderr=stderr),i) );
+      i = i + 1;
+    #ewhile
+
+    time.sleep(0.5);
+
+    running   = [ (j, k) for (j,k) in p if j.poll() == None ];
+    completed = [ (j, k) for (j,k) in p if j.poll() != None ];
+
+    for (j,k) in completed:
+      if j.returncode != 0:
+        retval = retval + j.returncode;
+        print "ERROR: Failed in cmd: %s" % cmd_list[k]; sys.stdout.flush();
+      else:
+        print "COMPLETED: cmd : %s" % cmd_list[k]; sys.stdout.flush();
+      #fi
+    #efor
+    p = running;
+    if len(p) == 0:
+      break;
+    #fi
+  #ewhile
+
+  return retval;
+#edef
+
+
+###############################################################################
+
+def run_seq_cmds(cmd_list, stdin=None, stdout=None, stderr=None):
+
+  for cmd in [ x for x in cmd_list if x ]:
+    retval = run_cmd(cmd, stdin=stdin, stdout=stdout, stderr=stderr);
+    if retval != 0:
+      print "ERROR: Failed on cmd: %s" % cmd;
+      return retval;
+    #fi
+    print "COMPLETED: cmd : %s" % cmd;
+    sys.stdout.flush();
+  #efor
+
+  return 0;
+#edef
+
+###############################################################################
+
+def run_cmd(cmd, bg=False, stdin=None, stdout=None, stderr=None):
+  p = subprocess.Popen(shlex.split(cmd), stdin=stdin, stdout=stdout, stderr=stderr);
+  if bg:
+    return p;
+  else:
+    (pid, r) = os.waitpid(p.pid, 0);
+    return r;
+  #fi
+#edef
+
+###############################################################################
+
+class PeekAheadFileReader(object):
+    def __init__(self, filename):
+        self.f = open_file(filename)
+        self.lines = []
+        self.fill_stack()
+        self.curLine = None
+        self.lineNr = 0
+ 
+    def skipWhite(self):
+        while(self.peekAhead().strip() == ''):
+            self.next()
+
+    def eof(self):
+        if not self.lines:
+            self.fill_stack()
+            return len(self.lines) == 0
+        
+        return False
+          
+    def tell(self):
+        return self.f.tell()
+
+    def reset(self, pos):
+        self.f.seek(pos)
+        self.lines = []
+        self.lineNr = 0 #fixme for positions > 0
+        self.curLine = None
+        self.fill_stack()
+
+    def __iter__(self):
+        return self
+
+    def fill_stack(self):
+        nlines = []
+        for i in range(100):
+            line = self.f.readline()
+            if(len(line) == 0):
+                break
+            nlines.append(line)
+        self.lines = nlines[::-1] + self.lines
+        return self.lines
+
+    def peekAhead(self):
+        if self.lines:
+            return self.lines[-1]
+        else:
+            return ''
+
+    def pushBack(self):
+        self.lines.append(self.curLine)
+        self.lineNr -= 1
+        self.curLine = None
+
+    def next(self):
+        if self.lines:
+            self.curLine =  self.lines.pop()
+            self.lineNr += 1
+            if not self.lines:
+               self.fill_stack()
+            return self.curLine
+        else:
+            raise StopIteration
+
+        
+        
+
+
+getNumber = re.compile('^([\d]+)')
+
+
+
